metadata_attributes = {
		"title": [],
		"creator": ["role", "file-as"],
		"subject": [],
		"description": [],
		"publisher": [],
		"contributor": ["role", "file-as"],
		"date": ["event"],
		"type": [],
		"format": [],
		"source": [],
		"language": [],
		"relation": [],
		"coverage": [],
		"identifier": ["scheme"],
		"rights": []
}

metadata_roles = [
	"abr",
	"acp",
	"act",
	"adi",
	"adp",
	"aft",
	"anl",
	"anm",
	"ann",
	"ant",
	"ape",
	"apl",
	"app",
	"aqt",
	"arc",
	"ard",
	"arr",
	"arr",
	"art",
	"asg",
	"asn",
	"ato",
	"att",
	"auc",
	"aud",
	"aui",
	"aus",
	"aut",
	"bdd",
	"bjd",
	"bkd",
	"bkd",
	"bkp",
	"blw",
	"bnd",
	"bpd",
	"brd",
	"brl",
	"bsl",
	"cas",
	"ccp",
	"chr",
	"cli",
	"cll",
	"clr",
	"clt",
	"cmm",
	"cmp",
	"cmt",
	"cnd",
	"cng",
	"cns",
	"coe",
	"col",
	"com",
	"con",
	"cor",
	"cos",
	"cot",
	"cou",
	"cov",
	"cpc",
	"cpe",
	"cph",
	"cpl",
	"cpt",
	"cre",
	"crp",
	"crr",
	"crr",
	"crt",
	"csl",
	"csp",
	"cst",
	"ctb",
	"cte",
	"ctg",
	"ctr",
	"cts",
	"ctt",
	"cur",
	"cwt",
	"dbp",
	"dfd",
	"dfe",
	"dft",
	"dgg",
	"dgs",
	"dis",
	"dln",
	"dnc",
	"dnr",
	"dpc",
	"dpt",
	"drm",
	"drt",
	"dsr",
	"dst",
	"dtc",
	"dte",
	"dtm",
	"dto",
	"dub",
	"edc",
	"edm",
	"edt",
	"egr",
	"elg",
	"elt",
	"eng",
	"enj",
	"etr",
	"evp",
	"exp",
	"fac",
	"fac",
	"fds",
	"fld",
	"flm",
	"fmd",
	"fmk",
	"fmo",
	"fmo",
	"fmp",
	"fnd",
	"fpy",
	"frg",
	"gis",
	"his",
	"hnr",
	"hst",
	"ill",
	"ill",
	"ilu",
	"ins",
	"ins",
	"inv",
	"isb",
	"itr",
	"ive",
	"ivr",
	"jud",
	"jug",
	"lbr",
	"lbt",
	"ldr",
	"led",
	"lee",
	"lel",
	"len",
	"let",
	"lgd",
	"lie",
	"lil",
	"lit",
	"lsa",
	"lse",
	"lso",
	"ltg",
	"lyr",
	"mcp",
	"mdc",
	"med",
	"mfp",
	"mfr",
	"mfr",
	"mod",
	"mon",
	"mrb",
	"mrk",
	"msd",
	"mte",
	"mtk",
	"mus",
	"nrt",
	"opn",
	"org",
	"orm",
	"osp",
	"oth",
	"own",
	"pan",
	"pat",
	"pbd",
	"pbl",
	"pdr",
	"pfr",
	"pht",
	"plt",
	"pma",
	"pmn",
	"pop",
	"ppm",
	"ppt",
	"pra",
	"prc",
	"prd",
	"pre",
	"prf",
	"prf",
	"prg",
	"prm",
	"prn",
	"pro",
	"prp",
	"prs",
	"prt",
	"prv",
	"pta",
	"pte",
	"ptf",
	"pth",
	"ptt",
	"pup",
	"rbr",
	"rcd",
	"rce",
	"rcp",
	"rdd",
	"red",
	"ren",
	"res",
	"rev",
	"rpc",
	"rps",
	"rpt",
	"rpy",
	"rse",
	"rsg",
	"rsp",
	"rsr",
	"rst",
	"rth",
	"rtm",
	"sad",
	"sce",
	"scl",
	"scr",
	"sds",
	"sec",
	"sgd",
	"sgn",
	"sht",
	"sll",
	"sng",
	"spk",
	"spn",
	"spy",
	"srv",
	"std",
	"stg",
	"stl",
	"stm",
	"stn",
	"str",
	"tcd",
	"tch",
	"ths",
	"tld",
	"tlp",
	"trc",
	"trl",
	"tyd",
	"tyg",
	"uvp",
	"vac",
	"vdg",
	"wac",
	"wal",
	"wam",
	"wat",
	"wdc",
	"wde",
	"win",
	"wit",
	"wpr",
	"wst"
]


